"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeParamsInUrlPlaceholdersAndParams = exports.splatSortings = exports.splatFilters = exports.splatSideloads = exports.requestActionType = void 0;

var _lodash = require("lodash");

/**
 * Raise meaningful errors in case the construction of the jsonapi is incomplete
 * and prevents the generation of the requestActionType for redux
 *
 * @param  {[type]} typePrefix  [description]
 * @param  {[type]} jsonapiType [description]
 * @return {[type]}             [description]
 */
const requestActionType = (typePrefix, jsonapiType) => {
  if (!typePrefix) {
    throw new Error('You need to set the action type (Create, update, etc.) of your resource !');
  }

  if (!jsonapiType) {
    throw new Error('You need to set the jsonapiType of your resource !');
  }

  return `${typePrefix.toUpperCase()}_${jsonapiType.toUpperCase()}_RESOURCE`;
};
/**
 * Resolve path dulications of sideloadings,
 * and make an array that is very easy to merge for json:api compliance
 *
 * @param { Object } startingSideloadPath - Current path to start from
 * @param { Object } nestedSideloads - nested sideloads to merge
 * @return { Array }
 *
 * @example
 *   splatSideloads('mainProfile.user', { avatar: true, profiles: { user: true } })
 *   # => ['mainProfile.user.avatar', 'mainProfile.user.profiles.user']
 */


exports.requestActionType = requestActionType;

const splatSideloads = (startingSideloadPath = '', nestedSideloads = {}) => {
  const currentSideloads = [];
  Object.keys(nestedSideloads).forEach(nestedKey => {
    const currentSideloadValue = nestedSideloads[nestedKey];
    let currentSideloadPath;

    if (startingSideloadPath === '') {
      currentSideloadPath = nestedKey;
    } else {
      currentSideloadPath = `${startingSideloadPath}.${nestedKey}`;
    }

    if (currentSideloadValue === true) {
      currentSideloads.push(currentSideloadPath);
    } else if ((0, _lodash.isObject)(currentSideloadValue)) {
      currentSideloads.push(splatSideloads(currentSideloadPath, currentSideloadValue));
    } else {
      throw new Error(`Sideloading only accepts true or nested objects as sideloads, but received ${currentSideloadValue}`);
    }
  });
  return (0, _lodash.flatten)(currentSideloads);
};
/**
 * Resolve path dulications of filters,
 * and make an array of filter/value(s) pairs that is very easy to merge for json:api compliance
 *
 * @param { String } startingFilterPath - Current path to start from
 * @param { Object } nestedFilters - nested filters to merge
 * @return { Array } of filter => Array value pair
 *
 * @example
 *   splatFilters('mainProfile.user',
 *     { avatar: [true], firstName: ['Martine', 'Albert,Marin'], notifications: { newsletter: true } }
 *   )
 *   # => [
 *     ['mainProfile[user][avatar]', [true]],
 *     ['mainProfile[user][firstName]', ['Martine', 'Albert%2CMarin']
 *     ['mainProfile[user][notifications][newsletter]', [true]
 *    ]
 */


exports.splatSideloads = splatSideloads;

const splatFilters = (startingFilterPath = '', nestedFilters = {}) => {
  let currentFilters = [];
  Object.keys(nestedFilters).forEach(nestedKey => {
    const currentFilterValue = nestedFilters[nestedKey];
    let currentFilterPath;

    if (startingFilterPath === '') {
      currentFilterPath = `filter[${nestedKey}]`;
    } else {
      currentFilterPath = `${startingFilterPath}[${nestedKey}]`;
    }

    if ((0, _lodash.isObject)(currentFilterValue) && !(0, _lodash.isArray)(currentFilterValue)) {
      currentFilters = currentFilters.concat(splatFilters(currentFilterPath, currentFilterValue, true));
    } else if ((0, _lodash.isString)(currentFilterValue) || (0, _lodash.isBoolean)(currentFilterValue) || (0, _lodash.isNumber)(currentFilterValue)) {
      currentFilters.push([currentFilterPath, [escapeFilterValue(currentFilterValue)]]);
    } else if ((0, _lodash.isArray)(currentFilterValue)) {
      currentFilters.push([currentFilterPath, currentFilterValue.map(v => escapeFilterValue(v))]);
    } else {
      throw new Error(`Filtering only accepts nested objects or strings or arrays as Filters, but received ${currentFilterValue}`);
    }
  });
  return currentFilters;
};
/**
 * Escape a filter value
 * Note : the json:api specs do not specify how to escape commas.
 * This implementation decides to escape them to their URL encoded value
 * Feel free to make a PR to allow reconfiguring jsonapi-orchestrator with a custom comma-transform
 * @param  {String,Boolean,Number} value
 * @return {String} commas escaped as %2C
 */


exports.splatFilters = splatFilters;

const escapeFilterValue = value => value.toString().replace(/,/g, '%2C');
/**
 * Assemble sorting paths,
 * and make an array of filter/value(s) pairs that is very easy to merge for json:api compliance
 *
 * @param { Array<Object> } sortings - List of sorting hashes
 * @return { Array } of sortings with +/- prefix
 *
 * @example
 *   splatSortings([ { avatar: [true] }, { notifications: { newsletter: true } }])
 *   # => ['-avatar', 'notifications[newsletter]']
 */


const splatSortings = sortings => sortings.map(sorting => {
  let [currentSortKey] = Object.keys(sorting);
  let currentSortPath = currentSortKey;
  let currentSortValueOrObject = sorting[currentSortKey];

  while ((0, _lodash.isObject)(currentSortValueOrObject)) {
    [currentSortKey] = Object.keys(currentSortValueOrObject);
    currentSortPath = `${currentSortPath}.${currentSortKey}`;
    currentSortValueOrObject = currentSortValueOrObject[currentSortKey];
  }

  if (currentSortValueOrObject === 'asc') {
    return currentSortPath;
  } else if (currentSortValueOrObject === 'desc') {
    return `-${currentSortPath}`;
  }

  throw new Error(`Sorting only accepts nested objects with ending values being strings 'asc' or 'desc', but received ${currentSortValueOrObject}`);
});
/**
 * @private
 *
 * Replace placeholder params in URLs
 *
 * @param url { String } url with placeholders
 * @param params { Object } params
 *
 * @return { Object } Object including new URL and replaced params
 *
 * @example
 *   replaceURlPlaceholders('/conversations/:id', { id 'cafebabe', otherParam: 'deadbeef' })
 *   # => { url: '/conversations/cafebabe', replacedParams: [:id] }
 */


exports.splatSortings = splatSortings;

const replaceUrlPlaceholders = (url, params) => {
  let mappedRoute = (0, _lodash.clone)(url);
  const replacedParams = [];
  (0, _lodash.forOwn)(params, (value, key) => {
    const placeholder = `:${key}`;

    if (mappedRoute.indexOf(placeholder) !== -1) {
      mappedRoute = mappedRoute.replace(placeholder, value.toString());
      replacedParams.push(key);
    }
  });
  return {
    url: mappedRoute,
    replacedParams
  };
};
/**
 * @private
 *
 * Merge params in URL params
 *
 * @example
 *
 *   mergeParamsInUrlParams('/conversation/:id?param1=1&param2=2', { param2: 2bis, param3: 3 })
 *   # => '/conversation/:id?param1=1&param2=2bis&param3=3'
 *
 */


const mergeParamsInUrlParams = (url, params) => {
  const originalSearchParams = url.split('?')[1] || ''; // TODO: @Dima polyfill URLSearchParams

  const paramsManager = new URLSearchParams(originalSearchParams);
  Object.keys(params).forEach(k => paramsManager.set(k, params[k]));
  const hasNoParams = paramsManager.keys().next().done;

  if (originalSearchParams) {
    return url.replace(`?${originalSearchParams}`, `?${paramsManager.toString()}`);
  } else if (hasNoParams) {
    return url;
  }

  return `${url}?${paramsManager.toString()}`;
};
/**
 * Replace URL placeholders by params when or available,
 * and add unmerged params as regular URL params
 *
 * @example
 *
 *   mergeParamsInUrlParams(
 *     '/conversation/:id?param1=1',
 *     { id: 'cafebabe', param2: 2bis, param3: 3 })
 *   )
 *   # => '/conversation/cafebabe?param1=1&param2=2bis&param3=3'
 *
 * @param {String} originalUrl
 * @param {Object} params
 * @return {String}
 */


const mergeParamsInUrlPlaceholdersAndParams = (originalUrl, params) => {
  const {
    url,
    replacedParams
  } = replaceUrlPlaceholders(originalUrl, params);
  return mergeParamsInUrlParams(url, (0, _lodash.omit)(params, replacedParams));
};

exports.mergeParamsInUrlPlaceholdersAndParams = mergeParamsInUrlPlaceholdersAndParams;